@@ -69,29 +69,27 @@
 
 QT_BEGIN_NAMESPACE
 
-const quint64 FRAME_SIZE_IN_BYTES = 512 * 512 * 2;	//maximum size of a frame when sending a message
+const quint64 MAX_OUTGOING_FRAME_SIZE_IN_BYTES = std::numeric_limits<int>::max() - 1;
+const quint64 DEFAULT_OUTGOING_FRAME_SIZE_IN_BYTES = 512 * 512 * 2; //default size of a frame when sending a message
 
 QWebSocketConfiguration::QWebSocketConfiguration() :
 #ifndef QT_NO_SSL
-    m_sslConfiguration(QSslConfiguration::defaultConfiguration()),
     m_ignoredSslErrors(),
     m_ignoreSslErrors(false),
 #endif
 #ifndef QT_NO_NETWORKPROXY
     m_proxy(QNetworkProxy::DefaultProxy),
 #endif
-    m_pSocket(Q_NULLPTR)
+    m_pSocket(nullptr)
 {
 }
 
 /*!
     \internal
 */
-QWebSocketPrivate::QWebSocketPrivate(const QString &origin, QWebSocketProtocol::Version version,
-                                     QWebSocket *pWebSocket) :
+QWebSocketPrivate::QWebSocketPrivate(const QString &origin, QWebSocketProtocol::Version version) :
     QObjectPrivate(),
-    q_ptr(pWebSocket),
-    m_pSocket(Q_NULLPTR),
+    m_pSocket(nullptr),
     m_errorString(),
     m_version(version),
     m_resourceName(),
@@ -109,21 +107,20 @@ QWebSocketPrivate::QWebSocketPrivate(const QString &origin, QWebSocketProtocol::
     m_closeCode(QWebSocketProtocol::CloseCodeNormal),
     m_closeReason(),
     m_pingTimer(),
-    m_dataProcessor(),
     m_configuration(),
     m_pMaskGenerator(&m_defaultMaskGenerator),
     m_defaultMaskGenerator(),
-    m_handshakeState(NothingDoneState)
+    m_handshakeState(NothingDoneState),
+    m_outgoingFrameSize(DEFAULT_OUTGOING_FRAME_SIZE_IN_BYTES)
 {
+    m_pingTimer.start();
 }
 
 /*!
     \internal
 */
-QWebSocketPrivate::QWebSocketPrivate(QTcpSocket *pTcpSocket, QWebSocketProtocol::Version version,
-                                     QWebSocket *pWebSocket) :
+QWebSocketPrivate::QWebSocketPrivate(QTcpSocket *pTcpSocket, QWebSocketProtocol::Version version) :
     QObjectPrivate(),
-    q_ptr(pWebSocket),
     m_pSocket(pTcpSocket),
     m_errorString(pTcpSocket->errorString()),
     m_version(version),
@@ -142,12 +139,13 @@ QWebSocketPrivate::QWebSocketPrivate(QTcpSocket *pTcpSocket, QWebSocketProtocol:
     m_closeCode(QWebSocketProtocol::CloseCodeNormal),
     m_closeReason(),
     m_pingTimer(),
-    m_dataProcessor(),
     m_configuration(),
     m_pMaskGenerator(&m_defaultMaskGenerator),
     m_defaultMaskGenerator(),
-    m_handshakeState(NothingDoneState)
+    m_handshakeState(NothingDoneState),
+    m_outgoingFrameSize(DEFAULT_OUTGOING_FRAME_SIZE_IN_BYTES)
 {
+    m_pingTimer.start();
 }
 
 /*!
@@ -158,6 +156,7 @@ void QWebSocketPrivate::init()
     Q_ASSERT(q_ptr);
     Q_ASSERT(m_pMaskGenerator);
 
+    m_dataProcessor->setParent(q_ptr);
     m_pMaskGenerator->seed();
 
     if (m_pSocket) {
@@ -228,6 +227,8 @@ bool QWebSocketPrivate::flush()
     return result;
 }
 
+#ifndef Q_OS_WASM
+
 /*!
     \internal
  */
@@ -244,6 +245,8 @@ qint64 QWebSocketPrivate::sendBinaryMessage(const QByteArray &data)
     return doWriteFrames(data, true);
 }
 
+#endif
+
 #ifndef QT_NO_SSL
 /*!
     \internal
@@ -267,6 +270,11 @@ QSslConfiguration QWebSocketPrivate::sslConfiguration() const
 void QWebSocketPrivate::ignoreSslErrors(const QList<QSslError> &errors)
 {
     m_configuration.m_ignoredSslErrors = errors;
+    if (Q_LIKELY(m_pSocket)) {
+        QSslSocket *pSslSocket = qobject_cast<QSslSocket *>(m_pSocket);
+        if (Q_LIKELY(pSslSocket))
+            pSslSocket->ignoreSslErrors(errors);
+    }
 }
 
 /*!
@@ -324,6 +332,8 @@ QWebSocket *QWebSocketPrivate::upgradeFrom(QTcpSocket *pTcpSocket,
     return pWebSocket;
 }
 
+#ifndef Q_OS_WASM
+
 /*!
     \internal
  */
@@ -334,19 +344,23 @@ void QWebSocketPrivate::close(QWebSocketProtocol::CloseCode closeCode, QString r
     if (!m_isClosingHandshakeSent) {
         Q_Q(QWebSocket);
         m_closeCode = closeCode;
-        m_closeReason = reason;
+        // 125 is the maximum length of a control frame, and 2 bytes are used for the close code:
+        const QByteArray reasonUtf8 = reason.toUtf8().left(123);
+        m_closeReason = QString::fromUtf8(reasonUtf8);
         const quint16 code = qToBigEndian<quint16>(closeCode);
         QByteArray payload;
         payload.append(static_cast<const char *>(static_cast<const void *>(&code)), 2);
-        if (!reason.isEmpty())
-            payload.append(reason.toUtf8());
+        if (!reasonUtf8.isEmpty())
+            payload.append(reasonUtf8);
         quint32 maskingKey = 0;
         if (m_mustMask) {
             maskingKey = generateMaskingKey();
-            QWebSocketProtocol::mask(payload.data(), payload.size(), maskingKey);
+            QWebSocketProtocol::mask(payload.data(), quint64(payload.size()), maskingKey);
         }
         QByteArray frame = getFrameHeader(QWebSocketProtocol::OpCodeClose,
-                                          payload.size(), maskingKey, true);
+                                          quint64(payload.size()), maskingKey, true);
+
+        Q_ASSERT(payload.length() <= 125);
         frame.append(payload);
         m_pSocket->write(frame);
         m_pSocket->flush();
@@ -376,11 +390,11 @@ void QWebSocketPrivate::open(const QNetworkRequest &request, bool mask)
     if (m_pSocket) {
         releaseConnections(m_pSocket);
         m_pSocket->deleteLater();
-        m_pSocket = Q_NULLPTR;
+        m_pSocket = nullptr;
     }
     //if (m_url != url)
     if (Q_LIKELY(!m_pSocket)) {
-        m_dataProcessor.clear();
+        m_dataProcessor->clear();
         m_isClosingHandshakeReceived = false;
         m_isClosingHandshakeSent = false;
 
@@ -412,7 +426,7 @@ void QWebSocketPrivate::open(const QNetworkRequest &request, bool mask)
                 setErrorString(message);
                 Q_EMIT q->error(QAbstractSocket::UnsupportedSocketOperationError);
             } else {
-                QSslSocket *sslSocket = new QSslSocket(q_ptr);
+                QSslSocket *sslSocket = new QSslSocket(q);
                 m_pSocket = sslSocket;
                 if (Q_LIKELY(m_pSocket)) {
                     m_pSocket->setSocketOption(QAbstractSocket::LowDelayOption, 1);
@@ -430,8 +444,9 @@ void QWebSocketPrivate::open(const QNetworkRequest &request, bool mask)
                         sslSocket->ignoreSslErrors(m_configuration.m_ignoredSslErrors);
     #ifndef QT_NO_NETWORKPROXY
                     sslSocket->setProxy(m_configuration.m_proxy);
+                    m_pSocket->setProtocolTag(QStringLiteral("https"));
     #endif
-                    sslSocket->connectToHostEncrypted(url.host(), url.port(443));
+                    sslSocket->connectToHostEncrypted(url.host(), quint16(url.port(443)));
                 } else {
                     const QString message = QWebSocket::tr("Out of memory.");
                     setErrorString(message);
@@ -441,7 +456,7 @@ void QWebSocketPrivate::open(const QNetworkRequest &request, bool mask)
         } else
     #endif
         if (url.scheme() == QStringLiteral("ws")) {
-            m_pSocket = new QTcpSocket(q_ptr);
+            m_pSocket = new QTcpSocket(q);
             if (Q_LIKELY(m_pSocket)) {
                 m_pSocket->setSocketOption(QAbstractSocket::LowDelayOption, 1);
                 m_pSocket->setSocketOption(QAbstractSocket::KeepAliveOption, 1);
@@ -452,8 +467,9 @@ void QWebSocketPrivate::open(const QNetworkRequest &request, bool mask)
                 setSocketState(QAbstractSocket::ConnectingState);
     #ifndef QT_NO_NETWORKPROXY
                 m_pSocket->setProxy(m_configuration.m_proxy);
+                m_pSocket->setProtocolTag(QStringLiteral("http"));
     #endif
-                m_pSocket->connectToHost(url.host(), url.port(80));
+                m_pSocket->connectToHost(url.host(), quint16(url.port(80)));
             } else {
                 const QString message = QWebSocket::tr("Out of memory.");
                 setErrorString(message);
@@ -468,6 +484,8 @@ void QWebSocketPrivate::open(const QNetworkRequest &request, bool mask)
     }
 }
 
+#endif
+
 /*!
     \internal
  */
@@ -478,7 +496,8 @@ void QWebSocketPrivate::ping(const QByteArray &payload)
     quint32 maskingKey = 0;
     if (m_mustMask)
         maskingKey = generateMaskingKey();
-    QByteArray pingFrame = getFrameHeader(QWebSocketProtocol::OpCodePing, payloadTruncated.size(),
+    QByteArray pingFrame = getFrameHeader(QWebSocketProtocol::OpCodePing,
+                                          quint64(payloadTruncated.size()),
                                           maskingKey, true);
     if (m_mustMask)
         QWebSocketProtocol::mask(&payloadTruncated, maskingKey);
@@ -556,18 +575,15 @@ void QWebSocketPrivate::enableMasking(bool enable)
 /*!
  * \internal
  */
-void QWebSocketPrivate::makeConnections(const QTcpSocket *pTcpSocket)
+void QWebSocketPrivate::makeConnections(QTcpSocket *pTcpSocket)
 {
     Q_ASSERT(pTcpSocket);
     Q_Q(QWebSocket);
 
     if (Q_LIKELY(pTcpSocket)) {
         //pass through signals
-        typedef void (QAbstractSocket:: *ASErrorSignal)(QAbstractSocket::SocketError);
-        typedef void (QWebSocket:: *WSErrorSignal)(QAbstractSocket::SocketError);
-        QObject::connect(pTcpSocket,
-                         static_cast<ASErrorSignal>(&QAbstractSocket::error),
-                         q, static_cast<WSErrorSignal>(&QWebSocket::error));
+        QObject::connect(pTcpSocket, &QAbstractSocket::errorOccurred,
+                         q, QOverload<QAbstractSocket::SocketError>::of(&QWebSocket::error));
 #ifndef QT_NO_NETWORKPROXY
         QObject::connect(pTcpSocket, &QAbstractSocket::proxyAuthenticationRequired, q,
                          &QWebSocket::proxyAuthenticationRequired);
@@ -575,8 +591,6 @@ void QWebSocketPrivate::makeConnections(const QTcpSocket *pTcpSocket)
         QObject::connect(pTcpSocket, &QAbstractSocket::readChannelFinished, q,
                          &QWebSocket::readChannelFinished);
         QObject::connect(pTcpSocket, &QAbstractSocket::aboutToClose, q, &QWebSocket::aboutToClose);
-        QObject::connect(pTcpSocket, &QAbstractSocket::bytesWritten, q, &QWebSocket::bytesWritten);
-
 
         QObjectPrivate::connect(pTcpSocket, &QObject::destroyed,
                                 this, &QWebSocketPrivate::socketDestroyed);
@@ -595,12 +609,11 @@ void QWebSocketPrivate::makeConnections(const QTcpSocket *pTcpSocket)
                              &QWebSocket::preSharedKeyAuthenticationRequired);
             QObject::connect(sslSocket, &QSslSocket::encryptedBytesWritten, q,
                              &QWebSocket::bytesWritten);
-            typedef void (QSslSocket:: *sslErrorSignalType)(const QList<QSslError> &);
             QObjectPrivate::connect(sslSocket,
-                                    static_cast<sslErrorSignalType>(&QSslSocket::sslErrors),
+                                    QOverload<const QList<QSslError>&>::of(&QSslSocket::sslErrors),
                                     this, &QWebSocketPrivate::_q_updateSslConfiguration);
             QObject::connect(sslSocket,
-                             static_cast<sslErrorSignalType>(&QSslSocket::sslErrors),
+                             QOverload<const QList<QSslError>&>::of(&QSslSocket::sslErrors),
                              q, &QWebSocket::sslErrors);
             QObjectPrivate::connect(sslSocket, &QSslSocket::encrypted,
                                     this, &QWebSocketPrivate::_q_updateSslConfiguration);
@@ -612,22 +625,26 @@ void QWebSocketPrivate::makeConnections(const QTcpSocket *pTcpSocket)
         }
     }
 
-    QObject::connect(&m_dataProcessor, &QWebSocketDataProcessor::textFrameReceived, q,
+    QObject::connect(m_dataProcessor, &QWebSocketDataProcessor::textFrameReceived, q,
                      &QWebSocket::textFrameReceived);
-    QObject::connect(&m_dataProcessor, &QWebSocketDataProcessor::binaryFrameReceived, q,
+    QObject::connect(m_dataProcessor, &QWebSocketDataProcessor::binaryFrameReceived, q,
                      &QWebSocket::binaryFrameReceived);
-    QObject::connect(&m_dataProcessor, &QWebSocketDataProcessor::binaryMessageReceived, q,
+    QObject::connect(m_dataProcessor, &QWebSocketDataProcessor::binaryMessageReceived, q,
                      &QWebSocket::binaryMessageReceived);
-    QObject::connect(&m_dataProcessor, &QWebSocketDataProcessor::textMessageReceived, q,
+    QObject::connect(m_dataProcessor, &QWebSocketDataProcessor::textMessageReceived, q,
                      &QWebSocket::textMessageReceived);
-    QObjectPrivate::connect(&m_dataProcessor, &QWebSocketDataProcessor::errorEncountered, this,
+    QObjectPrivate::connect(m_dataProcessor, &QWebSocketDataProcessor::errorEncountered, this,
                             &QWebSocketPrivate::close);
-    QObjectPrivate::connect(&m_dataProcessor, &QWebSocketDataProcessor::pingReceived, this,
+    QObjectPrivate::connect(m_dataProcessor, &QWebSocketDataProcessor::pingReceived, this,
                             &QWebSocketPrivate::processPing);
-    QObjectPrivate::connect(&m_dataProcessor, &QWebSocketDataProcessor::pongReceived, this,
+    QObjectPrivate::connect(m_dataProcessor, &QWebSocketDataProcessor::pongReceived, this,
                             &QWebSocketPrivate::processPong);
-    QObjectPrivate::connect(&m_dataProcessor, &QWebSocketDataProcessor::closeReceived, this,
+    QObjectPrivate::connect(m_dataProcessor, &QWebSocketDataProcessor::closeReceived, this,
                             &QWebSocketPrivate::processClose);
+
+    //fire readyread, in case we already have data inside the tcpSocket
+    if (pTcpSocket->bytesAvailable())
+        Q_EMIT pTcpSocket->readyRead();
 }
 
 /*!
@@ -637,7 +654,7 @@ void QWebSocketPrivate::releaseConnections(const QTcpSocket *pTcpSocket)
 {
     if (Q_LIKELY(pTcpSocket))
         pTcpSocket->disconnect();
-    m_dataProcessor.disconnect();
+    m_dataProcessor->disconnect();
 }
 
 /*!
@@ -711,6 +728,7 @@ QByteArray QWebSocketPrivate::getFrameHeader(QWebSocketProtocol::OpCode opCode,
                                              quint64 payloadLength, quint32 maskingKey,
                                              bool lastFrame)
 {
+    Q_Q(QWebSocket);
     QByteArray header;
     bool ok = payloadLength <= 0x7FFFFFFFFFFFFFFFULL;
 
@@ -744,7 +762,7 @@ QByteArray QWebSocketPrivate::getFrameHeader(QWebSocketProtocol::OpCode opCode,
         }
     } else {
         setErrorString(QStringLiteral("WebSocket::getHeader: payload too big!"));
-        Q_EMIT q_ptr->error(QAbstractSocket::DatagramTooLargeError);
+        Q_EMIT q->error(QAbstractSocket::DatagramTooLargeError);
     }
 
     return header;
@@ -763,11 +781,11 @@ qint64 QWebSocketPrivate::doWriteFrames(const QByteArray &data, bool isBinary)
     const QWebSocketProtocol::OpCode firstOpCode = isBinary ?
                 QWebSocketProtocol::OpCodeBinary : QWebSocketProtocol::OpCodeText;
 
-    int numFrames = data.size() / FRAME_SIZE_IN_BYTES;
+    int numFrames = data.size() / int(outgoingFrameSize());
     QByteArray tmpData(data);
     tmpData.detach();
     char *payload = tmpData.data();
-    quint64 sizeLeft = quint64(data.size()) % FRAME_SIZE_IN_BYTES;
+    quint64 sizeLeft = quint64(data.size()) % outgoingFrameSize();
     if (Q_LIKELY(sizeLeft))
         ++numFrames;
 
@@ -776,7 +794,7 @@ qint64 QWebSocketPrivate::doWriteFrames(const QByteArray &data, bool isBinary)
     if (Q_UNLIKELY(numFrames == 0))
         numFrames = 1;
     quint64 currentPosition = 0;
-    quint64 bytesLeft = data.size();
+    quint64 bytesLeft = quint64(data.size());
 
     for (int i = 0; i < numFrames; ++i) {
         quint32 maskingKey = 0;
@@ -786,7 +804,7 @@ qint64 QWebSocketPrivate::doWriteFrames(const QByteArray &data, bool isBinary)
         const bool isLastFrame = (i == (numFrames - 1));
         const bool isFirstFrame = (i == 0);
 
-        const quint64 size = qMin(bytesLeft, FRAME_SIZE_IN_BYTES);
+        const quint64 size = qMin(bytesLeft, outgoingFrameSize());
         const QWebSocketProtocol::OpCode opcode = isFirstFrame ? firstOpCode
                                                                : QWebSocketProtocol::OpCodeContinue;
 
@@ -960,24 +978,39 @@ void QWebSocketPrivate::processHandshake(QTcpSocket *pSocket)
     case ReadingStatusState:
         if (!pSocket->canReadLine())
             return;
-        m_statusLine = pSocket->readLine();
+        m_statusLine = pSocket->readLine().trimmed();
         if (Q_UNLIKELY(!parseStatusLine(m_statusLine, &m_httpMajorVersion, &m_httpMinorVersion, &m_httpStatusCode, &m_httpStatusMessage))) {
             errorDescription = QWebSocket::tr("Invalid statusline in response: %1.").arg(QString::fromLatin1(m_statusLine));
             break;
         }
         m_handshakeState = ReadingHeaderState;
         Q_FALLTHROUGH();
-    case ReadingHeaderState:
+    case ReadingHeaderState: {
+        // TODO: this should really use the existing code from QHttpNetworkReplyPrivate::parseHeader
+        auto lastHeader = m_headers.end();
         while (pSocket->canReadLine()) {
             QString headerLine = readLine(pSocket);
-            const QStringList headerField = headerLine.split(QStringLiteral(": "),
-                                                             QString::SkipEmptyParts);
-            if (headerField.size() == 2) {
-                m_headers.insertMulti(headerField[0].toLower(), headerField[1]);
-            }
-            if (headerField.isEmpty()) {
+
+            if (headerLine.isEmpty()) {
+                // end of headers
                 m_handshakeState = ParsingHeaderState;
                 break;
+            } else if (headerLine.startsWith(QLatin1Char(' ')) || headerLine.startsWith(QLatin1Char('\t'))) {
+                // continuation line -- add this to the last header field
+                if (Q_UNLIKELY(lastHeader == m_headers.end())) {
+                    errorDescription = QWebSocket::tr("Malformed header in response: %1.").arg(headerLine);
+                    break;
+                }
+                lastHeader.value().append(QLatin1Char(' '));
+                lastHeader.value().append(headerLine.trimmed());
+            } else {
+                int colonPos = headerLine.indexOf(QLatin1Char(':'));
+                if (Q_UNLIKELY(colonPos <= 0)) {
+                    errorDescription = QWebSocket::tr("Malformed header in response: %1.").arg(headerLine);
+                    break;
+                }
+                lastHeader = m_headers.insert(headerLine.left(colonPos).trimmed().toLower(),
+                                              headerLine.mid(colonPos + 1).trimmed());
             }
         }
 
@@ -989,6 +1022,7 @@ void QWebSocketPrivate::processHandshake(QTcpSocket *pSocket)
             return;
         }
         Q_FALLTHROUGH();
+    }
     case ParsingHeaderState: {
         const QString acceptKey = m_headers.value(QStringLiteral("sec-websocket-accept"), QString());
         const QString upgrade = m_headers.value(QStringLiteral("upgrade"), QString());
@@ -1014,7 +1048,7 @@ void QWebSocketPrivate::processHandshake(QTcpSocket *pSocket)
                 if (!ok)
                     errorDescription =
                       QWebSocket::tr("Accept-Key received from server %1 does not match the client key %2.")
-                            .arg(acceptKey).arg(accept);
+                            .arg(acceptKey, accept);
             } else {
                 errorDescription =
                     QWebSocket::tr("QWebSocketPrivate::processHandshake: Invalid statusline in response: %1.")
@@ -1023,8 +1057,7 @@ void QWebSocketPrivate::processHandshake(QTcpSocket *pSocket)
         } else if (m_httpStatusCode == 400) {
             //HTTP/1.1 400 Bad Request
             if (!version.isEmpty()) {
-                const QStringList versions = version.split(QStringLiteral(", "),
-                                                           QString::SkipEmptyParts);
+                const QStringList versions = version.split(QStringLiteral(", "), Qt::SkipEmptyParts);
                 if (!versions.contains(QString::number(QWebSocketProtocol::currentVersion()))) {
                     //if needed to switch protocol version, then we are finished here
                     //because we cannot handle other protocols than the RFC one (v13)
@@ -1074,9 +1107,6 @@ void QWebSocketPrivate::processStateChanged(QAbstractSocket::SocketState socketS
     Q_ASSERT(m_pSocket);
     Q_Q(QWebSocket);
     QAbstractSocket::SocketState webSocketState = this->state();
-    int port = 80;
-    if (m_request.url().scheme() == QStringLiteral("wss"))
-        port = 443;
 
     switch (socketState) {
     case QAbstractSocket::ConnectedState:
@@ -1093,16 +1123,17 @@ void QWebSocketPrivate::processStateChanged(QAbstractSocket::SocketState socketS
                 headers << qMakePair(QString::fromLatin1(key),
                                      QString::fromLatin1(m_request.rawHeader(key)));
 
-            const QString handshake =
-                    createHandShakeRequest(m_resourceName,
-                                           m_request.url().host()
-                                                % QStringLiteral(":")
-                                                % QString::number(m_request.url().port(port)),
-                                           origin(),
-                                           QString(),
-                                           QString(),
-                                           m_key,
-                                           headers);
+            const auto format = QUrl::RemoveScheme | QUrl::RemoveUserInfo
+                                | QUrl::RemovePath | QUrl::RemoveQuery
+                                | QUrl::RemoveFragment;
+            const QString host = m_request.url().toString(format).mid(2);
+            const QString handshake = createHandShakeRequest(m_resourceName,
+                                                             host,
+                                                             origin(),
+                                                             QString(),
+                                                             QString(),
+                                                             m_key,
+                                                             headers);
             if (handshake.isEmpty()) {
                 m_pSocket->abort();
                 Q_EMIT q->error(QAbstractSocket::ConnectionRefusedError);
@@ -1131,8 +1162,6 @@ void QWebSocketPrivate::processStateChanged(QAbstractSocket::SocketState socketS
         //do nothing
         //to make C++ compiler happy;
         break;
-    default:
-        break;
     }
 }
 
@@ -1140,7 +1169,7 @@ void QWebSocketPrivate::socketDestroyed(QObject *socket)
 {
     Q_ASSERT(m_pSocket);
     if (m_pSocket == socket)
-        m_pSocket = Q_NULLPTR;
+        m_pSocket = nullptr;
 }
 
 /*!
@@ -1153,10 +1182,10 @@ void QWebSocketPrivate::processData()
     while (m_pSocket->bytesAvailable()) {
         if (state() == QAbstractSocket::ConnectingState) {
             if (!m_pSocket->canReadLine())
-                break;
+                return;
             processHandshake(m_pSocket);
-        } else {
-            m_dataProcessor.process(m_pSocket);
+        } else if (!m_dataProcessor->process(m_pSocket)) {
+            return;
         }
     }
 }
@@ -1170,7 +1199,10 @@ void QWebSocketPrivate::processPing(const QByteArray &data)
     quint32 maskingKey = 0;
     if (m_mustMask)
         maskingKey = generateMaskingKey();
-    m_pSocket->write(getFrameHeader(QWebSocketProtocol::OpCodePong, data.size(), maskingKey, true));
+    m_pSocket->write(getFrameHeader(QWebSocketProtocol::OpCodePong,
+                                    unsigned(data.size()),
+                                    maskingKey,
+                                    true));
     if (data.size() > 0) {
         QByteArray maskedData = data;
         if (m_mustMask)
@@ -1277,6 +1309,80 @@ void QWebSocketPrivate::setSocketState(QAbstractSocket::SocketState state)
     }
 }
 
+/*!
+    \internal
+ */
+void QWebSocketPrivate::setMaxAllowedIncomingFrameSize(quint64 maxAllowedIncomingFrameSize)
+{
+    m_dataProcessor->setMaxAllowedFrameSize(maxAllowedIncomingFrameSize);
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::maxAllowedIncomingFrameSize() const
+{
+    return m_dataProcessor->maxAllowedFrameSize();
+}
+
+/*!
+    \internal
+ */
+void QWebSocketPrivate::setMaxAllowedIncomingMessageSize(quint64 maxAllowedIncomingMessageSize)
+{
+    m_dataProcessor->setMaxAllowedMessageSize(maxAllowedIncomingMessageSize);
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::maxAllowedIncomingMessageSize() const
+{
+    return m_dataProcessor->maxAllowedMessageSize();
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::maxIncomingMessageSize()
+{
+    return QWebSocketDataProcessor::maxMessageSize();
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::maxIncomingFrameSize()
+{
+    return QWebSocketDataProcessor::maxFrameSize();
+}
+
+/*!
+    \internal
+ */
+void QWebSocketPrivate::setOutgoingFrameSize(quint64 outgoingFrameSize)
+{
+    if (outgoingFrameSize <= maxOutgoingFrameSize())
+        m_outgoingFrameSize = outgoingFrameSize;
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::outgoingFrameSize() const
+{
+    return m_outgoingFrameSize;
+}
+
+/*!
+    \internal
+ */
+quint64 QWebSocketPrivate::maxOutgoingFrameSize()
+{
+    return MAX_OUTGOING_FRAME_SIZE_IN_BYTES;
+}
+
+
 /*!
     \internal
  */
@@ -1425,6 +1531,7 @@ void QWebSocketPrivate::setReadBufferSize(qint64 size)
         m_pSocket->setReadBufferSize(m_readBufferSize);
 }
 
+#ifndef Q_OS_WASM
 /*!
     \internal
  */
@@ -1433,5 +1540,6 @@ bool QWebSocketPrivate::isValid() const
     return (m_pSocket && m_pSocket->isValid() &&
             (m_socketState == QAbstractSocket::ConnectedState));
 }
+#endif
 
 QT_END_NAMESPACE
